/**
 * Copyright (C) 2003-2004 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.spap.launcher;

import java.io.File;
import java.io.IOException;


import sync4j.syncclient.spap.AssetInstallationException;
import sync4j.syncclient.spap.installer.InstallationContext;
import sync4j.syncclient.spds.util.StaticDataHelper;

/**
 * This class represents a <code>Laucher</code> for the execution of
 * the win program.
 *
 * @version  $Id: Win32Launcher.java,v 1.1.1.1 2004/04/09 11:23:04 fabius Exp $
 * @author Stefano Nichele
 */

public class Win32Launcher implements Launcher {


    // -----------------------------------------------------------Public methods

    /**
     * Executes the win program with given name.
     *
     * @param programName the program to execute.
     * @param install <code>true</code> if the program is the installation program,
     *  <code>false</code> if the program is the uninstallation program
     * @param ctx installation context information
     *
     * @throws AssetInstallationException if an error occurs during the execution
     */
    public int execute(String programName, boolean install, InstallationContext ctx)
            throws AssetInstallationException {

        String completeProgramPath = ctx.getWorkingDirectory() + File.separator + programName;
        StaticDataHelper.log("Win32Launcher - Execute " +
                             completeProgramPath);
        int exitState = -1;
        try {
            Process proc = Runtime.getRuntime().exec(
                    completeProgramPath, null, new File(ctx.getWorkingDirectory())
                    );

            proc.waitFor();

            exitState = proc.exitValue();

            StaticDataHelper.log("Process executed. Return state: " + exitState);

        } catch (Exception e) {
            throw new AssetInstallationException("Error launching '" + completeProgramPath + "'", e);
        }

        return exitState;
    }


}