/**
 * Copyright (C) 2003-2004 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.spds.engine;

import java.util.Hashtable;
import java.util.Enumeration;

import java.lang.reflect.*;

import sync4j.syncclient.spds.*;
import sync4j.syncclient.spds.util.*;
import sync4j.syncclient.spds.source.SPSSyncSource;
import sync4j.syncclient.common.SimpleClassLoader;

/**
 * This is a factory for SyncSource objects. Its istantiates the right instance
 * given the source to be used. The it calls the setXXX() method for each
 * property in the source definition <i>properties</i> hashtable.
 *
 * @autor Fabio Maggi
 * @author Stefano Fornari
 *
 * @version $Id: SyncSourceFactory.java,v 1.1.1.1 2004/04/09 11:23:04 fabius Exp $
 **/
public class SyncSourceFactory {

    private Hashtable syncSourcesDefinition;
    private Hashtable syncSources;

    private String libDir = null;

    /**
     * Creates a new SyncSourceFactory reading the sync source implementation
     * classes from the given <i>libDirectory</i> and using the given
     * <i>syncSourcesDefinition</i>.
     *
     * @param libDir directory where classes are looked for
     * @param syncSourcesDefinition sources definitions
     *
     */
    public SyncSourceFactory (String libDir, Hashtable syncSourcesDefinition) {
        this.syncSourcesDefinition = syncSourcesDefinition;
        this.syncSources = new Hashtable();
        this.libDir = libDir;
    }

    /**
     * @return syncSource
     */
    public SyncSource getSyncSource(String dataStoreName) {

        SyncSource s = (SyncSource)syncSources.get(dataStoreName);

        if (s != null) {
            return s;
        }

        String syncSourceClass = null;
        SyncSource syncSource = null;

        SyncSourceDefinition def = (SyncSourceDefinition) syncSourcesDefinition.get(dataStoreName);

        syncSourceClass = def.getSourceClass();

        try {
            syncSource = (SyncSource) (Class.forName(syncSourceClass)).newInstance();
        } catch (Exception e) {
            e.printStackTrace();
            StaticDataHelper.log( "Error loading class "
                                + syncSourceClass
                                + ": "
                                + e.getMessage()
                                );
            return null;
        }

        //
        // If we loaded a SPSSyncSource, we set its classLoader property to
        // the SimpleClassLoader instance.
        //

        if (syncSourceClass.endsWith(".SPSSyncSource")) {
            ((SPSSyncSource)syncSource).setClassLoader(new SimpleClassLoader(this.libDir));
        }


        Hashtable properties = def.getProperties();

        Enumeration e = properties.keys();

        String key = null;
        while (e.hasMoreElements()) {
            key = (String)e.nextElement();
            setProperty(syncSource, key, (String)properties.get(key));
        }

        syncSources.put((Object) dataStoreName, (Object) syncSource);

        return syncSource;
    }

    // ---------------------------------------------------------- Private methds

    private void setProperty(SyncSource source, String key, String value) {
        String methodName = "";

        char firstLetter = key.toUpperCase().charAt(0);
        if (key.length() > 1) {
            methodName = key.substring(1);
        }

        methodName = "set" + firstLetter + methodName;

        Class sourceClass = source.getClass();

        try {
            Method m = sourceClass.getMethod(methodName, new Class[] { String.class });
            m.invoke(source, new String[] {value});
        } catch (Exception e) {
            StaticDataHelper.log("Property "
                                + key
                                + " not set to "
                                + value
                                + ". Method "
                                + methodName
                                + "(String s) not found in "
                                + sourceClass.getName()
                                );
        }
    }
}