/**
 * Copyright (C) 2003-2004 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

/**
 * This class create DataStore
 * and provide method to call new record
 *
 * @author Fabio Maggi @ Funambol
 * $Id: DataStoreImpl.java,v 1.1.1.1 2004/04/09 11:23:04 fabius Exp $
 */

package sync4j.syncclient.sps.j2me;

import java.util.Vector;

import java.util.Date;

import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordEnumeration;

import sync4j.syncclient.sps.common.*;
import sync4j.syncclient.sps.common.util.*;




public class DataStoreImpl extends DataStore {

    public DataStoreImpl(String dataStoreName, RecordMetadata recordMetadata) {
        super(dataStoreName, recordMetadata);
    }

    /**
     * create record
     *
     * @param key
     * @return record
     **/
    public Record newRecord(String key) {

        Record record = null;
        record = new Record(this, key);
        return record;

    }

    //----------------------------------------------------------- Public methods

    /**
     * read record
     * @param Record
     **/
    public Record readRecord(Record record) {

        RecordStore recordStore = null;

        SimpleFilter simpleFilter = null;

        RecordEnumeration recordEnumeration = null;

        StrParser st  = null;

        String dataRecord = null;

        int i = 0;


        simpleFilter = new SimpleFilter(record.getKey() + "|");

        try {

            recordStore = RecordStore.openRecordStore(this.getDataStoreName(), true);

            recordEnumeration = recordStore.enumerateRecords(simpleFilter, null, false);

            while (recordEnumeration.hasNextElement()) {
                    dataRecord = new String(recordEnumeration.nextRecord());
            }

            st = new StrParser(dataRecord.substring(dataRecord.indexOf("|") + 1), "|!;!|");

            while (st.hasMoreElements()) {
                if(i != record.getPositionKeyField()) {
                    record.setField(i, st.nextElement());
                }
                i++;
            }

            recordStore.closeRecordStore();


        } catch (javax.microedition.rms.InvalidRecordIDException e1) {
            StaticDataHelper.log("error:" + e1.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e2) {
            StaticDataHelper.log("error:" + e2.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e3) {
            StaticDataHelper.log("error:" + e3.getMessage());
        }

        return record;

    }


    /**
     * store record
     * @param record
     **/
    public Record storeRecord(Record record) {

        RecordStore recordStore = null;

        SimpleFilter simpleFilter = null;

        RecordEnumeration recordEnumeration = null;

        simpleFilter = new SimpleFilter(record.getKey() + "|");

        String dataRecord = null;

        int idRecord = -1;

        try {

            recordStore = RecordStore.openRecordStore(this.getDataStoreName(), true);

            recordEnumeration = recordStore.enumerateRecords(simpleFilter, null, false);

            while (recordEnumeration.hasNextElement()) {

                idRecord = recordEnumeration.nextRecordId();

            }


            dataRecord = record.getKey() + "|";

            for (int i=0; i < record.getLength(); i++) {
                if(i != record.getPositionKeyField()) {
                    dataRecord = dataRecord + (String) record.getString(i) + "|!;!|";
                }
            }

            if (idRecord != -1) {
                recordStore.setRecord(idRecord, dataRecord.getBytes(), 0, (dataRecord.getBytes()).length);
            } else {
                recordStore.addRecord(dataRecord.getBytes(), 0, (dataRecord.getBytes()).length);
            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e1) {
            StaticDataHelper.log("error:" + e1.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e2) {
            StaticDataHelper.log("error:" + e2.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e3) {
            StaticDataHelper.log("error:" + e3.getMessage());
        }


        return record;

    }


    /**
     * delete record
     * @param record
     **/
    public void deleteRecord(Record record) {


        RecordStore recordStore = null;

        SimpleFilter simpleFilter = null;

        RecordEnumeration recordEnumeration = null;

        simpleFilter = new SimpleFilter(record.getKey() + "|");

        String dataRecord = null;

        int idRecord = -1;

        try {

            recordStore = RecordStore.openRecordStore(this.getDataStoreName(), true);

            recordEnumeration = recordStore.enumerateRecords(simpleFilter, null, false);

            while (recordEnumeration.hasNextElement()) {

                idRecord = recordEnumeration.nextRecordId();

            }


            if (idRecord == -1) {
                //errore record non esistente
                return;
            } else {

                recordStore.deleteRecord(idRecord);
            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e1) {
            StaticDataHelper.log("error:" + e1.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e2) {
            StaticDataHelper.log("error:" + e2.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e3) {
            StaticDataHelper.log("error:" + e3.getMessage());
        }

    }



    /**
     * return alla records of dataStore
     *
     * @return find records
     **/
    public Vector findAllRecords() {

        Vector records = new Vector();

        RecordStore recordStore = null;

        SimpleComparator simpleComparator = null;

        RecordEnumeration recordEnumeration = null;

        Record record = null;

        StrParser strp = null;

        String key = null;

         try {

            recordStore = RecordStore.openRecordStore(this.getDataStoreName(), true);

            simpleComparator = new SimpleComparator();

            recordEnumeration = recordStore.enumerateRecords(null, simpleComparator, false);

            while (recordEnumeration.hasNextElement()) {

                strp = new StrParser(new String(recordEnumeration.nextRecord()), "|");

                key = strp.nextElement();

                record = new Record(this, key);

                records.addElement(record);

            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e1) {
            StaticDataHelper.log("error:" + e1.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e2) {
            StaticDataHelper.log("error:" + e2.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e3) {
            StaticDataHelper.log("error:" + e3.getMessage());
        }

        return records;

    }



    /**
     * return records of dataStore find by RecordFilter
     *
     * @param RecordFilter filter
     * @return find records
     **/
    public Vector findRecords(RecordFilter recordFilter) {

        Vector allRecords  = null;
        Vector findRecords = new Vector();

        Record record = null;

        allRecords = this.findAllRecords();

        for(int i=0; i < allRecords.size(); i++) {

            record = this.readRecord((Record) allRecords.elementAt(i));

            if(recordFilter.accept(record)) {
                findRecords.addElement(allRecords.elementAt(i));
            }
        }

        return findRecords;

    }

    public long getNextKey() {
        //da implementare
        return 0;
    }



    /**
     * return a Vector of Records
     * found by state, last timestamp
     *
     * @param state state of record
     * @param since last timestamp
     * @return find records
     **/
    public Vector findRecords(char state, Date since) {

        return null;

    }


    /**
     * Method define start DB operathions
     */
    public void startDBOperations() {

    }

    /**
     * Method define end DB operathions
     */
    public void commitDBOperations() {

    }

}